@@ -10,20 +10,24 @@
 #include "efxc2Utils.h"
 #include "efxc2Compiler.h"
 
-void Compiler::initializeDefines() {
-    numDefines = 1;
-    defines = new D3D_SHADER_MACRO[numDefines];
-    defines[numDefines - 1].Name = nullptr;
-    defines[numDefines - 1].Definition = nullptr;
-}
-
 void Compiler::Compile() {
-    //Default output variable name
-    if (variableName == nullptr) {
-        variableName = setupVariableName(model, entryPoint);
-    }
+
+    auto SourceCode = params.get_SourceCode();
+    auto SourceLen = params.get_SourceLen();
+    auto eflags = params.get_eflags();
+    auto sflags = params.get_sflags();
+    auto secondary_flags = params.get_secondary_flags();
+    auto numDefines = params.get_numDefines();
+    std::string _entryPoint = params.get_entryPoint();
+    const char* entryPoint = _entryPoint.c_str();
+    std::string _model = params.get_model();
+    const char* model = _model.c_str();
+    std::string _inputFile = params.get_inputFile();
+    const char* inputFile = _inputFile.c_str();
+    auto defines = params.get_defines();
+
     ID3DBlob* errors = nullptr;
-    if (verbose) {
+    if (params.get_verbose()) {
         printf("Calling D3DCompile2(\n");
 
         printf("\t SourceCode,\n");
@@ -79,7 +83,7 @@ void Compiler::Compile() {
 #endif
     auto ptr = api.get_ptr_D3DCompile2();
     HRESULT hr = ptr(
-        SourceCode,
+        SourceCode.get()->data(),
         SourceLen,
         inputFile,
         defines,
@@ -115,9 +119,10 @@ void Compiler::Compile() {
 void Compiler::Disassemble() {
     auto const* compiledString = (unsigned char*)compilerOutput->GetBufferPointer();
     size_t compiledLen = compilerOutput->GetBufferSize();
+    auto disassembly_flags = params.get_disassembly_flags();
     HRESULT hr = 0;
     disassemblyCodeBlob = nullptr;
-    if (verbose) {
+    if (params.get_verbose()) {
         printf("Calling D3DDisassemble(\n");
         printf("\t compiledString,\n");
         printf("\t %zu,\n", compiledLen);
@@ -149,10 +154,11 @@ void Compiler::Disassemble() {
 void Compiler::StripShader() {
     auto const* compiledString = (unsigned char*)compilerOutput->GetBufferPointer();
     size_t compiledLen = compilerOutput->GetBufferSize();
+    auto strip_flags = params.get_strip_flags();
     HRESULT hr = 0;
     strippedBlob = nullptr;
     if (strip_flags != 0) {
-        if (verbose) {
+        if (params.get_verbose()) {
             printf("Calling D3DStripShader(\n");
             printf("\t compiledString,\n");
             printf("\t %zu,\n", compiledLen);
@@ -195,6 +201,14 @@ size_t Compiler::WriteAssemblyCode(FILE* f) {
 }
 
 size_t Compiler::WriteIncludeFile(FILE* f) {
+    auto variableName = params.get_variableName();
+    
+    //Default output variable name
+    if (variableName == "") {
+        std::string model = params.get_model();
+        std::string entryPoint = params.get_entryPoint();
+        variableName = setupVariableName(model.c_str(), entryPoint.c_str());
+    } 
     unsigned char const* outputString = nullptr;
     size_t outputLen = 0;
     if (strippedBlob == nullptr) {
@@ -205,7 +219,7 @@ size_t Compiler::WriteIncludeFile(FILE* f) {
         outputString = (unsigned char*)strippedBlob->GetBufferPointer();
         outputLen = strippedBlob->GetBufferSize();
     }
-    WriteByteArrayConst(f, outputString, outputLen, variableName, outputHex);
+    WriteByteArrayConst(f, outputString, outputLen, variableName.c_str(), params.get_outputHex());
     return outputLen;
 }
 
@@ -224,12 +238,12 @@ size_t Compiler::WriteObjectFile(FILE* f) {
     return outputLen;
 }
 
-char* Compiler::GetPDBFileName() {
+std::string Compiler::GetPDBFileName() {
     auto const* compiledString = (unsigned char*)compilerOutput->GetBufferPointer();
     size_t compiledLen = compilerOutput->GetBufferSize();
     HRESULT hr = 0;
     /*Get filename*/
-    if (verbose) {
+    if (params.get_verbose()) {
         printf("Calling D3DGetBlobPart(\n");
         printf("\t compiledString,\n");
         printf("\t %zu,\n", compiledLen);
@@ -257,32 +271,33 @@ char* Compiler::GetPDBFileName() {
     return pName;
 }
 
-void Compiler::SetPDBFileName(_In_ const char* _fileName) {
+void Compiler::SetPDBFileName(_In_ const std::string& _fileName) {
     // Blobs are always a multiple of 4 bytes long. Since DxilShaderDebugName
     // is itself 4 bytes, we pad the storage of the string (not the string itself)
     // to 4 bytes also.
-    size_t fileNameLen = strlen(_fileName);
+    size_t fileNameLen = _fileName.length();
 
     size_t lengthOfNameStorage = (fileNameLen + 0x3) & ~0x3;
     size_t nameBlobPartSize = sizeof(ShaderDebugName) + lengthOfNameStorage;
-    auto pNameBlobContent = (ShaderDebugName*)(malloc(nameBlobPartSize));
 
-    if (pNameBlobContent == nullptr) {
-        fprintf(stderr, "malloc failed/n");
-        print_errno();
-    }
+    std::vector<char> pNameBlobContent;
+    pNameBlobContent.resize(nameBlobPartSize + sizeof(ShaderDebugName));
     // Ensure bytes after name are indeed zeroes:
-    memset(pNameBlobContent, 0, nameBlobPartSize);
-    pNameBlobContent->Flags = 0;
+    std::fill(pNameBlobContent.begin(), pNameBlobContent.end(), 0);
+    auto* header = reinterpret_cast<ShaderDebugName*>(&pNameBlobContent[0]);
+
+    header->Flags = 0;
     // declared length does not include the null terminator:
-    pNameBlobContent->NameLength = (uint16_t)fileNameLen - 1;
+    header->NameLength = (uint16_t)fileNameLen - 1;
     // but the null terminator is expected to be present:
-    memcpy(pNameBlobContent + 1, _fileName, fileNameLen);
+    for (int i = 0; i < _fileName.length(); i++) {
+        pNameBlobContent[sizeof(ShaderDebugName) + i] = _fileName[i];
+    }
 
     auto const* compiledString = (unsigned char*)compilerOutput->GetBufferPointer();
     size_t compiledLen = compilerOutput->GetBufferSize();
 
-    if (verbose) {
+    if (params.get_verbose()) {
         printf("Calling D3DSetBlobPart(\n");
         printf("\t compiledString,\n");
         printf("\t %zu,\n", compiledLen);
@@ -294,7 +309,7 @@ void Compiler::SetPDBFileName(_In_ const char* _fileName) {
     }
     HRESULT hr;
     auto ptr = api.get_ptr_D3DSetBlobPart();
-    hr = ptr(compiledString, compiledLen, D3D_BLOB_DEBUG_NAME, 0, pNameBlobContent,
+    hr = ptr(compiledString, compiledLen, D3D_BLOB_DEBUG_NAME, 0, pNameBlobContent.data(),
         nameBlobPartSize, &pShaderWithNewName);
     /*
     HRESULT D3DSetBlobPart(
@@ -324,7 +339,7 @@ size_t Compiler::WritePDBFile(FILE* f) {
         compiledString = (unsigned char*)pShaderWithNewName->GetBufferPointer();
         compiledLen = pShaderWithNewName->GetBufferSize();
     }
-    if (verbose) {
+    if (params.get_verbose()) {
         printf("Calling D3DGetBlobPart(\n");
         printf("\t compiledString,\n");
         printf("\t %zu,\n", compiledLen);
@@ -350,21 +365,3 @@ size_t Compiler::WritePDBFile(FILE* f) {
     fwrite(outputString, outputLen, 1, f);
     return compiledLen;
 }
-
-void Compiler::LoadSourceCode(FILE* f) {
-    readall(f, &SourceCode, &SourceLen);
-}
-
-void Compiler::add_define(char* defineOption) {
-    assert(defineOption == nullptr);
-    numDefines++;
-    //Copy the old array into the new array, but put the new definition at the beginning
-    auto newDefines = new D3D_SHADER_MACRO[numDefines];
-    for (size_t i = 1; i < numDefines; i++) {
-        newDefines[i] = defines[i - 1];
-    }
-    delete[] defines;
-    defines = newDefines;
-    defines[0].Name = defineOption;
-    defines[0].Definition = "1";
-}
