@@ -13,6 +13,8 @@
 #define EFXC2UTILS_H_INCLUDED
 #include "efxc2.h"
 
+#define M_BUFFER   std::shared_ptr<std::vector<char>>
+
 /*from: https://stackoverflow.com/questions/14002954/c-programming-how-to-read-the-whole-file-contents-into-a-buffer */
 /* Size of each input chunk to be
    read and allocate for. */
@@ -69,26 +71,24 @@ const std::array <ProfilePrefix, PROFILE_PREFIX_TABLE_LEN>g_profilePrefixTable =
   { "vs_3_sw", "g_vs3ff"},
 } };
 
-char* concat(const char* s1, const char* s2);
+std::string GetFileName(_In_ const std::string& path, _Out_ int* IsSpecialFolder);
 #ifdef _WIN32
-wchar_t* concat(const wchar_t* s1, const wchar_t* s2);
-#endif
-char* GetFileName(_In_ char* path, _Out_ int* IsSpecialFolder);
-#ifdef _WIN32
-wchar_t* GetFileName(_In_ wchar_t* path, _Out_ int* IsSpecialFolder);
-void FixupFileName(_Inout_ wchar_t* FileName);
-bool parseOpt(_In_ const wchar_t* option, 
-    _In_ int argc, 
-    _In_ wchar_t* argv[1], 
-    _Inout_ int* index, 
-    _Inout_opt_ wchar_t** argumentOption);
+std::wstring GetFileName(_In_ const std::wstring& path, _Out_ int* IsSpecialFolder);
+void FixupFileName(_Inout_ std::string& FileName);
+void FixupFileName(_Inout_ std::wstring& FileName);
+bool parseOpt(_In_ const wchar_t* option,
+    _In_ int argc,
+    _In_ wchar_t* argv[1],
+    _Inout_ int* index,
+    _Inout_opt_ std::wstring* argumentOption);
+
 #else  /* _WIN32 */
 char* LoadSource(_In_ const char* filename, _Out_ size_t* len);
-bool parseOpt(_In_ const char* option, 
-    _In_ int argc, 
-    _In_ char* argv[1], 
-    _Inout_	int* index, 
-    _Inout_opt_ char** argumentOption);
+bool parseOpt(_In_ const char* option,
+    _In_ int argc,
+    _In_ char* argv[1],
+    _Inout_ int* index,
+    _Inout_opt_ std::string * argumentOption);
 #endif /* _WIN32 */
 void print_copyright();
 [[noreturn]] void print_errno(errno_t _errno);
@@ -100,15 +100,13 @@ void print_unsupported_arg_help();
 [[noreturn]] void print_usage_toomany();
 [[noreturn]] void print_version();
 [[noreturn]] void print_windows_error();
-int readall(_In_ FILE* in, 
-    _Out_writes_bytes_(*sizeptr) char** dataptr, 
-    _Out_opt_ size_t* sizeptr);
-char* setupVariableName(_In_ const char* model,
-    const _In_ char* entryPoint);
+int readall(_In_ FILE* in,
+    _Out_ M_BUFFER& dataptr);
+std::string setupVariableName(_In_ const std::string& model,
+    _In_ std::string const& entryPoint);
 #ifdef _WIN32
-wchar_t* utf8_decode(const char* str, size_t nbytes);
-char* utf8_encode(const wchar_t* wstr, size_t nchars);
-char* utf8_encode(const wchar_t* wstr);
+std::wstring utf8_decode(const std::string& str);
+std::string utf8_encode(const std::wstring& wstr);
 #endif
 void WriteByteArrayConst(_In_ FILE* f, 
     _In_reads_bytes_(len) const unsigned char* outString, 
